<?php

namespace Pinpoint\PackingSlips\Model\Pdf;

use Magento\Framework\App\Filesystem\DirectoryList;

/**
 * Class GenerateSlip
 * @package Pinpoint\PackingSlips\Model\Pdf
 */
class GenerateSlip extends \Magento\Framework\Model\AbstractModel
{
    /**
     * @var \Dompdf\Dompdf
     */
    protected $domPdf;

    /**
     * @var \Magento\Sales\Api\OrderRepositoryInterface
     */
    protected $orderRepository;

    /**
     * @var \Magento\Catalog\Api\ProductRepositoryInterfaceFactory
     */
    protected $productRepository;

    /**
     * @var \Magento\Framework\Pricing\Helper\Data
     */
    protected $currencyHelper;

    /**
     * @var \Magento\Framework\View\LayoutFactory
     */
    protected $layoutFactory;

    /**
     * @var \Magento\Framework\Filesystem\DirectoryList
     */
    protected $directoryList;

    /**
     * @var string
     */
    protected $fontFolder;

    /**
     * @var string
     */
    protected $fontCacheFolder;

    /**
     * GenerateSlip constructor.
     * @param \Magento\Sales\Api\OrderRepositoryInterface $orderRepository
     * @param \Magento\Framework\Pricing\Helper\Data $currencyHelper
     * @param \Magento\Catalog\Api\ProductRepositoryInterfaceFactory $productRepositoryFactory
     * @param \Magento\Framework\Filesystem\DirectoryList $dir
     * @param \Magento\Framework\View\LayoutFactory $layoutFactory
     * @param \Magento\Framework\Filesystem $filesystem
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource|null $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb|null $resourceCollection
     * @param array $data
     */
    public function __construct(
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \Magento\Framework\Pricing\Helper\Data $currencyHelper,
        \Magento\Catalog\Api\ProductRepositoryInterfaceFactory $productRepositoryFactory,
        \Magento\Framework\Filesystem\DirectoryList $dir,
        \Magento\Framework\View\LayoutFactory $layoutFactory,
        \Magento\Framework\Filesystem $filesystem,
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $mediaDir = DIRECTORY_SEPARATOR . trim($filesystem->getDirectoryRead(DirectoryList::MEDIA)->getAbsolutePath(), '/');
        $fontFolder = $mediaDir . DIRECTORY_SEPARATOR . 'dompdf_fonts';
        $fontCacheFolder = $mediaDir . DIRECTORY_SEPARATOR . 'dompdf_fonts_cache';

        if (!is_dir($fontFolder)) {
            mkdir($fontFolder, 0777, true);
        }

        if (!is_dir($fontCacheFolder)) {
            mkdir($fontCacheFolder, 0777, true);
        }

        $this->fontCacheFolder = $fontCacheFolder;
        $this->fontFolder = $fontFolder;
        $this->directoryList = $dir;
        $this->orderRepository = $orderRepository;
        $this->productRepository = $productRepositoryFactory;
        $this->currencyHelper = $currencyHelper;
        $this->layoutFactory = $layoutFactory;

        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    public function generateFromApi($orderId)
    {
        $this->_appState->emulateAreaCode('adminhtml', [$this, "generateSlip"], [$orderId, true]);
    }

    /**
     * @param $orderId
     * @param bool $save
     * @param bool $returnSort
     * @return array|\Dompdf\Dompdf
     */
    public function generateSlip($orderId, $save = false, $returnSort = false)
    {
        $slip = $this->domPdf = new \Dompdf\Dompdf([
            'enable_remote' => true,
            'DOMPDF_ENABLE_CSS_FLOAT' => true,
            'defaultFont' => 'Helvetica',
            'fontDir' => $this->fontFolder,
            'fontCache' => $this->fontCacheFolder
        ]);

        try {
            $order = $this->orderRepository->get($orderId);
        } catch (\Exception $ex) {
            return null;
        }

        $pdfData = [
            'order' => [
                'id' => $orderId,
                'increment_id' => $order->getIncrementId(),
                'date_purchased' => $order->getCreatedAt(),
                'payment_method' => $order->getPayment()->getData('additional_information')['method_title'],
                'discount' => $order->getDiscountAmount(),
                'barcode' => $orderId,
                'delivery_method' => strtolower($order->getShippingDescription()),
                'has_logo' => false
            ],
            'customer' => [
                'customer_name' => sprintf('%s %s', $order->getShippingAddress()->getFirstname(), $order->getShippingAddress()->getLastname()),
                'billing_address' => $order->getBillingAddress(),
                'delivery_address' => $order->getShippingAddress(),
            ],
            'order_items' => $order->getItems()
        ];

        $requiredMachines = [];

        /** @var \Magento\Sales\Api\Data\OrderItemInterface $item */
        foreach ($pdfData['order_items'] as $item) {
            try {
                $product = $this->productRepository->create()->getById($item->getProductId());
            } catch (\Exception $exception) {
                return null;
            }
            $pdfData['products'][$item->getProductId()] = $product;
            $requiredMachines[] = $productMachines = $product->getResource()->getAttribute('required_machines')->getFrontend()->getValue($product);
        }

        //sort items to put personalised options last
        usort($pdfData['order_items'], function ($a, $b) use ($pdfData)
        {
            $productAOptions = $a->getProductOptions();
            $productBOptions = $b->getProductOptions();

            if ($productAOptions && $productBOptions) {
                if (isset($productAOptions['options']) && isset($productBOptions['options'])) {
                    $machineA = $pdfData['products'][$a->getProductId()]->getResource()->getAttribute('required_machines')
                        ->getFrontend()->getValue($pdfData['products'][$a->getProductId()]);

                    $machineB = $pdfData['products'][$b->getProductId()]->getResource()->getAttribute('required_machines')
                        ->getFrontend()->getValue($pdfData['products'][$b->getProductId()]);

                    if ($machineA && $machineB) {
                        if ($machineA == $machineB) {
                            return 0;
                        } else {
                            return 1;
                        }
                    }

                    return 0;
                } else if (isset($productAOptions['options']) && !isset($productBOptions['options'])) {
                    return 1;
                } else {
                    return -1;
                }
            } else if ($productAOptions && !$productBOptions) {
                return 1;
            } else {
                return -1;
            }
        });

        $pdfData['order']['required_machines'] = rtrim(implode(',', array_filter($requiredMachines, 'strlen')), ',');
        $pdfData['order']['id'] = str_pad($pdfData['order']['id'], 9, 0, STR_PAD_LEFT);

        foreach ($pdfData['order_items'] as $item) {
            if (!isset($item->getProductOptions()['options'])) {
                $pdfData['product_pages']['simple'][] = $item;
            } else {
                foreach ($item->getProductOptions()['options'] as $option) {
                    if ($option['option_type'] === 'file') {
                        $pdfData['order']['has_logo'] = true;
                        break;
                    }
                }

                $machine = $pdfData['products'][$item->getProductId()]->getResource()->getAttribute('required_machines')
                    ->getFrontend()->getValue($pdfData['products'][$item->getProductId()]);

                if ($machine) {
                    $pdfData['product_pages'][$machine][] = $item;
                } else {
                    $pdfData['product_pages']['na'][] = $item;
                }
            }
        }

        foreach ($pdfData['product_pages'] as $prodType => $products) {
            if ($prodType !== 'simple') {
                usort($pdfData['product_pages'][$prodType], function ($a, $b) use ($pdfData) {
                    $itemTypeA = $pdfData['products'][$a->getProductId()]->getResource()->getAttribute('ppp_item_type')
                    ->getFrontend()->getValue($pdfData['products'][$a->getProductId()]);

                    $itemTypeB = $pdfData['products'][$b->getProductId()]->getResource()->getAttribute('ppp_item_type')
                        ->getFrontend()->getValue($pdfData['products'][$b->getProductId()]);

                    if ($itemTypeA === $itemTypeB) {
                        return 0;
                    } else {
                        return 1;
                    }
                });
            }
        }

        $counter = [];
        foreach ($pdfData['product_pages'] as $machine => $product_page) {
            if (!isset($counter[$machine])) {
                $counter[$machine] = 0;
            }

            $counter[$machine] += 1;
        }

        if (count($counter) > 1) {
          $machineSort = 'Split Machine Orders';
        } else {
          $machineSort = array_keys($counter, max($counter))[0];
        }

        $slip->loadHtml($this->getPdfTemplate($pdfData));
        $slip->render();

        if ($save) {
            $pdfOutput = $slip->output();
            $outFolder = $this->directoryList->getPath('var') . DIRECTORY_SEPARATOR .
                'export' . DIRECTORY_SEPARATOR .
                'slips' . DIRECTORY_SEPARATOR;

            if (!is_dir($outFolder)) {
                mkdir($outFolder);
            }

            \file_put_contents(
                $outFolder .
                "packing-slip-{$order->getId()}.pdf",
                $pdfOutput
            );
        }

        if (!$save) {
          try {
              $order->setState('printed')->setStatus('printed');
              $this->orderRepository->save($order);
          } catch (\Exception $exception) {
              unset($exception);
          }
        }

        $this->domPdf = null;

        if (!$returnSort) {
            return $slip;
        } else {
            return [$slip, $machineSort, $order];
        }
    }

    /**
     * @param $text
     * @return \Dompdf\Dompdf
     */
    public function createSplitterPage($text)
    {
        $pdf = new \Dompdf\Dompdf([
            'enable_remote' => true,
            'DOMPDF_ENABLE_CSS_FLOAT' => true,
            'defaultFont' => 'Helvetica',
            'fontDir' => $this->fontFolder,
            'fontCache' => $this->fontCacheFolder
        ]);;

        $pdf->loadHtml("<h1>Machine: $text</h1>");
        $pdf->render();

        return $pdf;
    }

    /**
     * @param $params
     * @return mixed
     */
    protected function getPdfTemplate($params)
    {
        return $this->layoutFactory->create()
            ->createBlock('Pinpoint\PackingSlips\Block\Adminhtml\Pdf\PackingSlip')
            ->setData($params)
            ->toHtml()
        ;
    }
}
